import copy

from direct.gui.OnscreenImage import OnscreenImage
from direct.gui.OnscreenText import OnscreenText
from direct.interval.LerpInterval import LerpPosInterval, LerpHprInterval
from direct.interval.MetaInterval import Sequence
from direct.showbase.PythonUtil import clamp
from direct.showbase.ShowBase import ShowBase
from panda3d.core import *
from panda3d.core import KeyboardButton, PerlinNoise, WindowProperties, TransparencyAttrib, TextureStage, TextNode
import random
import sys

def main(base, SANDBOX, MAP_SIZE):
    global screw_charge
    global _action
    global _next_position
    global current_direction
    global _ticks_since_column
    global now_playing
    global screw_charge

    base.accept('escape', sys.exit)


    load_prc_file("config.prc")

    loading_image = OnscreenImage("loading.png")
    loading_image.reparent_to(base.render2d)
    base.graphics_engine.render_frame()
    base.graphics_engine.render_frame()
    base.graphics_engine.render_frame()
    base.graphics_engine.render_frame()
    loading_image.destroy()

    BEAT_LENGTH = 0.6

    START_HEIGHT = 8
    BEAT_FALLOFF = 0.015625  # 1/64
    SEGMENT_HEIGHT = 0.7
    COLUMN_FREQ = 8
    SCREW_CHARGE_REQ = 3
    TILES_PER_BLOCK = 3

    _tile_offset = 0 if TILES_PER_BLOCK % 2 else 0.5

    left_button = 'a'
    right_button = 'd'
    drop_button = 'space'
    pump_button = 'f'
    speed_key = KeyboardButton.ascii_key(b'r')

    smnts = base.loader.load_model("model/segments")
    smnts.setDepthOffset(-1)
    straight_segment = smnts.find("**/straight")
    right_segment = smnts.find("**/right")
    left_segment = smnts.find("**/left")
    ground_segment = smnts.find("**/ground")
    waterfall = smnts.find("**/waterfall"); waterfall.reparent_to(base.render)
    waterfall_segment = copy.copy(waterfall)
    waterdrop_segment = smnts.find("**/waterdrop")
    road_segment = smnts.find("**/road")
    spill = smnts.find("**/spill"); spill.reparent_to(waterfall); spill.set_pos(0, -1, 0)
    column = smnts.find("**/column")

    tank_1 = base.loader.load_model("model/tank_1")
    tank_1_empty = base.loader.load_model("model/tank_1_empty")

    screw_segment = base.loader.load_model("model/screw")

    city = base.loader.load_model("model/city_0")
    city_empty = base.loader.load_model("model/city_0_empty")
    city_tank = base.loader.load_model("model/city_tank")
    city_tank_empty = base.loader.load_model("model/city_tank_empty")
    city_tank_empty.find("**/beacon").set_transparency(TransparencyAttrib.M_alpha)
    city_tank_empty.find("**/beacon").set_alpha_scale(0.2)

    screw_anchor = NodePath('anchor')
    screw_anchor.reparent_to(screw_segment)
    _screw = screw_segment.find("**/screw")
    _screw.reparent_to(screw_anchor)
    LerpHprInterval(_screw, 0.5, (360, 0, 0), (0, 0, 0)).loop()
    screw_anchor.set_hpr(0, -14, 0)


    n = 0
    e = 1
    s = 2
    w = 3
    current_direction = n

    dir_vec_dict = {
        n: LVector3f(0, 1, 0),
        e: LVector3f(1, 0, 0),
        s: LVector3f(0, -1, 0),
        w: LVector3f(-1, 0, 0),
    }

    dir_h_dict = {
        n: 0,
        e: 270,
        s: 180,
        w: 90
    }


    screw_charge = 0

    screw_marker_images = [base.loader.load_texture("pump_progress/" + str(i) + ".png") for i in range(4)]

    screw_marker = OnscreenImage(screw_marker_images[0], pos=(60, 0, -160), parent=base.pixel2d, scale=(50, 0, 150))
    if SANDBOX:
        screw_marker.hide()
    controls = OnscreenText("A/D - turn left/right\nSpace - drop a bit\nR - speed up\nH - hide this tooltip" +
                            ("\nF - Archimedes' screw" if SANDBOX else "") + "\nesc - abandon your post",
                            pos=(base.a2dRight-0.5, base.a2dTop-0.1), parent=base.aspect2d,
                            font=base.loader.load_font("spqr.ttf"), fg=(1, 1, 1, 1), bg=(0.4, 0.3, 0, 0.2))
    base.accept("h", controls.hide)

    music = [
        base.loader.loadMusic("music/LD36_vox_flute.wav"),
        base.loader.loadMusic("music/LD36_vox_flute.wav"),
        base.loader.loadMusic("music/LD36.wav"),
        base.loader.loadMusic("music/LD36_short_nomar.wav"),
        base.loader.loadMusic("music/LD36_short_nomar.wav"),
        base.loader.loadMusic("music/LD36_norec.wav"),
        base.loader.loadMusic("music/LD36_norec.wav"),
        base.loader.loadMusic("music/LD36_vox.wav"),
    ]
    now_playing = music[0]
    clock = ClockObject()
    def shuffle_music(task):
        global now_playing
        if now_playing.status() != now_playing.PLAYING:
            now_playing = random.choice(music)
            now_playing.play()
            clock.reset()
        return task.cont

    base.task_mgr.add(shuffle_music, "shuffle")

    # region setup
    base.set_background_color(VBase4(0.588, 0.686, 0.8, 1))

    props = WindowProperties()
    props.setCursorHidden(True)
    base.win.request_properties(props)

    dl = DirectionalLight('dl')
    dl.set_color(VBase4(1, 1, 1, 1))
    dl.set_shadow_caster(True, 2048, 2048)
    dl.get_lens().set_film_size(64, 64)
    dl.get_lens().set_near_far(50, 400)
    dlnp = base.render.attach_new_node(dl)
    dlnp.reparent_to(base.render)
    dlnp.set_pos(128, 128, 90)
    dlnp.look_at(0, 0, 0)
    base.render.set_light(dlnp)

    backlight = DirectionalLight('bl')
    backlight.set_color(VBase4(0.2, 0.2, 0.2, 1))
    blnp = base.render.attach_new_node(backlight)
    blnp.set_pos(0, 0, 0)
    blnp.look_at(128, 128, 90)
    base.render.set_light(blnp)


    al = AmbientLight('al')
    al.set_color(VBase4(0.1, 0.05, 0, 1))
    alnp = base.render.attach_new_node(al)
    base.render.set_light(alnp)

    base.disable_mouse()
    camera_pivot = NodePath('camera pivot')
    camera_pivot.reparent_to(base.render)
    base.camera.reparent_to(camera_pivot)
    base.camera.set_pos(0, -10, 20)
    base.camera.set_p(-60)

    cam_focus = NodePath('focus')
    cam_focus.reparent_to(camera_pivot)
    base.camera.reparent_to(cam_focus)
    def move_camera(task):
        def center_mouse():
            if base.mouseWatcherNode.has_mouse():
                base.win.move_pointer(0,
                    int(base.win.getXSize() / 2.0),
                    int(base.win.getYSize() / 2.0))
        # confirm a mouse exists
        if not base.mouseWatcherNode.has_mouse():
            print("No mouse found.")
            return task.cont

        dt = globalClock.getDt()  # todo consider using dt in camera control

        look_speed_h = 900
        look_speed_v = 1000
        min_camera_p = -25
        max_camera_p = 50

        if abs(base.mouseWatcherNode.getMouseX()) > 0.001 or abs(base.mouseWatcherNode.getMouseY()) > 0.001:
            cam_focus.set_h(cam_focus.get_h() - (base.mouseWatcherNode.getMouseX() * look_speed_h*dt))
            cam_focus.setP(
                clamp(cam_focus.getP() + (base.mouseWatcherNode.getMouseY() * look_speed_v*dt),
                      min_camera_p, max_camera_p
                      )
            )

        # recenter the mouse manually because M_relative doesn't want to
        center_mouse()

        return task.cont

    base.task_mgr.add(move_camera, "move camera")

    # region world

    sky = base.loader.load_model('model/skybox')
    sky.reparent_to(base.render)
    sky.set_scale(MAP_SIZE, MAP_SIZE, 32)
    sky.set_hpr(0, 0, 0)
    sky.set_pos(MAP_SIZE/2-1.5, MAP_SIZE/2-1.5, 0)
    sky.set_material(Material())
    sky.set_tex_scale(TextureStage.get_default(), 1, MAP_SIZE*16)
    sky.set_transparency(TransparencyAttrib.M_alpha)
    sky.set_alpha_scale(0.5)

    perlin = PerlinNoise2()

    heightmap = [[perlin.noise(x*0.05, y*0.05)*10 for x in range(MAP_SIZE//TILES_PER_BLOCK)] for y in range(MAP_SIZE//TILES_PER_BLOCK)]
    tank_map = [[False for _ in range(MAP_SIZE)] for _ in range(MAP_SIZE)]

    ground = NodePath('ground')
    ground_segments = [[]]
    filled_tanks = []
    full_tanks = []
    empty_tank_locs = []

    max_height = -100
    for y in range(len(heightmap)):
        gs = []
        for x in range(len(heightmap[y])):
            new_seg = ground.attach_new_node("ground " + str(x) + str(y))
            h = heightmap[y][x]
            if h > max_height:
                max_height = h
            new_seg.set_pos(x*TILES_PER_BLOCK+_tile_offset, y*TILES_PER_BLOCK+_tile_offset, h)
            new_seg.set_scale(TILES_PER_BLOCK, TILES_PER_BLOCK, 1)
            ground_segment.instance_to(new_seg)
            gs.append(new_seg)

            # chance for tank
            if random.random() < 0.03:
                new_tank = base.render.attach_new_node("tank")
                new_tank.set_pos(x*TILES_PER_BLOCK, y*TILES_PER_BLOCK, heightmap[y][x])
                tank_1_empty.instance_to(new_tank)
                tank_map[y][x] = new_tank
                new_tank.set_h(random.choice(list(dir_h_dict.values())))
                empty_tank_locs.append((x, y))
        ground_segments.append(gs)

    random.shuffle(empty_tank_locs)
    sky.set_z(max_height-16)

    lake = copy.copy(spill)
    lake.reparent_to(base.render)
    lake.set_scale(3*TILES_PER_BLOCK, TILES_PER_BLOCK, 8)
    lake_x, lake_y = MAP_SIZE//2, MAP_SIZE//16
    lake_z = heightmap[(lake_y) // TILES_PER_BLOCK][(lake_x) // TILES_PER_BLOCK]
    lake.set_pos(lake_x + _tile_offset, lake_y + _tile_offset, lake_z)

    lake_screws = []
    for i in range(5):
        new_screw = base.render.attach_new_node('new screw')
        screw_segment.instance_to(new_screw)
        new_screw.set_pos(lake.get_pos() + LVector3(i*2, 2, 3) + LVector3(-5 + _tile_offset, _tile_offset, 0))
        lake_screws.append(new_screw)

    # generate cities
    cities = []  # (city_tank, city)
    if not SANDBOX:

        for i in range(3):
            if i == 0:
                ct_x, ct_y = MAP_SIZE // 2, MAP_SIZE - MAP_SIZE // 16
            elif i == 1:
                ct_x, ct_y = MAP_SIZE // 16, MAP_SIZE//2
            elif i == 2:
                ct_x, ct_y = MAP_SIZE - MAP_SIZE // 16, MAP_SIZE//2


            city_tank_z = heightmap[ct_y // TILES_PER_BLOCK][ct_x // TILES_PER_BLOCK]
            new_ct = base.render.attach_new_node("ct")
            city_tank_empty.instance_to(new_ct)
            new_ct.set_pos(ct_x + _tile_offset, ct_y + _tile_offset, city_tank_z)
            districts = []
            for x in range(-3*TILES_PER_BLOCK, 3*TILES_PER_BLOCK+1, TILES_PER_BLOCK):
                for y in range(-4*TILES_PER_BLOCK, 4*TILES_PER_BLOCK+1, TILES_PER_BLOCK):
                    if x == 0 and y == 0:
                        continue
                    if random.random() < 0.25:
                        continue
                    city_x, city_y = ct_x + x, ct_y + y
                    new_city = base.render.attach_new_node("city")
                    city_empty.instance_to(new_city)
                    new_city.reparent_to(base.render)
                    try:
                        city_z = heightmap[city_y // TILES_PER_BLOCK][city_x // TILES_PER_BLOCK]
                    except IndexError:
                        new_city.remove_node()
                        continue
                    new_city.set_pos(city_x, city_y, city_z)
                    new_city.set_h(random.random() * 360)
                    districts.append(new_city)
            cities.append((new_ct, districts))

    def fill_city(i):
        t, cs = cities.pop(i)
        new_t = NodePath("city tank")
        new_t.reparent_to(base.render)
        city_tank.instance_to(new_t)
        new_t.set_pos(t.get_pos())
        new_t.set_hpr(t.get_hpr())
        t.removeNode()
        for c in cs:
            city.instance_to(c)
        global screw_charge
        screw_charge = SCREW_CHARGE_REQ
        screw_marker.setImage(screw_marker_images[3])
        if SANDBOX:
            screw_marker.hide()


    def fill_tank(x, y):
        global screw_charge
        screw_charge += 1
        screw_marker.setImage(screw_marker_images[clamp(screw_charge, 0, 3)])
        if SANDBOX:
            screw_marker.hide()
        tank = tank_map[y][x]
        tank_1.instance_to(tank)
        filled_tanks.append(tank)

    ground.flatten_strong()
    ground.reparent_to(base.render)

    aqueduct_segment_heights = [[[] for _ in range(MAP_SIZE)] for _ in range(MAP_SIZE)]
    column_map = [[False for _ in range(MAP_SIZE)] for _ in range(MAP_SIZE)]

    _next_position = Point3(MAP_SIZE/2, MAP_SIZE/2, START_HEIGHT)


    _action = None
    def set_action(button):
        global _action
        _action = button

    base.accept(right_button, set_action, [right_button])
    base.accept(left_button, set_action, [left_button])
    base.accept(drop_button, set_action, [drop_button])
    base.accept(pump_button, set_action, [pump_button])

    segments = []
    columns = []
    _ticks_since_column = 64

    aqueduct = NodePath("aqueduct")
    aqueduct.reparent_to(base.render)

    def build_segment(task):
        global screw_charge
        global _action
        global _next_position
        global current_direction
        global _ticks_since_column
        _ticks_since_column += 1
        this_position = _next_position
        this_h = dir_h_dict[current_direction]
        is_down = base.mouseWatcherNode.is_button_down
        if _action == left_button or is_down(KeyboardButton.ascii_key(b'a')):
            # go left
            next_segment = left_segment
            current_direction = (current_direction - 1) % 4
        elif _action == right_button or is_down(KeyboardButton.ascii_key(b'd')):
            # go right
            next_segment = right_segment
            current_direction = (current_direction + 1) % 4
        else:
            # go straight
            next_segment = straight_segment

        if (_action == pump_button or is_down(KeyboardButton.ascii_key(b'f'))) and (screw_charge >= SCREW_CHARGE_REQ or SANDBOX):
            screw_charge = 0
            screw_marker.setImage(screw_marker_images[0])
            if SANDBOX:
                screw_marker.hide()
            this_position += LVector3(0, 0, 4)
            _ticks_since_column = COLUMN_FREQ - 1
            next_segment = screw_segment
        elif _action == drop_button or is_down(KeyboardButton.space()):
            new_waterfall = aqueduct.attach_new_node("waterfall")
            new_waterfall.set_pos(this_position)
            new_waterfall.set_h(this_h)
            waterdrop_segment.instance_to(new_waterfall)
            this_position += LVector3(0, 0, -SEGMENT_HEIGHT)
            _ticks_since_column = 64

        new_seg = aqueduct.attach_new_node("segment " + str(len(segments)))
        if next_segment == screw_segment:
            new_seg.reparent_to(base.render)
        segments.append(new_seg)
        new_seg.set_pos(this_position)
        new_seg.set_h(this_h)
        next_segment.instance_to(new_seg)

        # build columns if needed
        if _ticks_since_column >= COLUMN_FREQ:
            new_col = base.render.attach_new_node("column" + str(len(columns)))
            columns.append(new_col)
            new_col.set_pos(this_position)
            column.instance_to(new_col)
            _ticks_since_column = 0
            # x, y = (round(this_position.get_x()) + 1) // 4, (round(this_position.get_y() + 1 // 4):
            x, y = round(this_position.get_x()), round(this_position.get_y())
            column_map[y][x] = True

        # attach the waterfall
        waterfall.set_pos(this_position)
        waterfall.set_h(dir_h_dict[(current_direction+2) % 4])
        x, y = round(spill.get_x(base.render)), round(spill.get_y(base.render))
        try:
            spill.set_z(base.render, heightmap[(y+1)//TILES_PER_BLOCK][(x+1)//TILES_PER_BLOCK] + 0.1)
        except IndexError:
            pass

        # set up the next position
        _next_position = this_position + dir_vec_dict[current_direction] - LVector3(0, 0, BEAT_FALLOFF)

        # check for tanks
        x, y = (round(_next_position.get_x()) + 1) // TILES_PER_BLOCK, (round(_next_position.get_y()) + 1) // TILES_PER_BLOCK
        try:
            tank = tank_map[y][x]
        except IndexError:
            tank = False
        if tank and tank not in filled_tanks:
            relative_pos = tank.get_relative_point(base.render, _next_position)
            if (round(relative_pos.x), round(relative_pos.y)) in [(-1, 0), (1, 0), (0, 0), (0, 1), (0, -1)]:
                fill_tank(x, y)
                new_wf = base.render.attach_new_node('waterfall')
                new_wf.set_pos(waterfall.get_pos())
                new_wf.set_hpr(waterfall.get_hpr())
                waterfall_segment.instance_to(new_wf)
                collide()
                print("filled tank :D")
                return task.done

        #check for cities
        for i in range(len(cities)):
            city_tank = cities[i][0]
            relative_pos = city_tank.get_relative_point(base.render, _next_position)
            if (round(relative_pos.x), round(relative_pos.y)) in [(-1, 0), (1, 0), (0, 0), (0, 1), (0, -1)]:
                fill_city(i)
                new_wf = base.render.attach_new_node('waterfall')
                new_wf.set_pos(waterfall.get_pos())
                new_wf.set_hpr(waterfall.get_hpr())
                waterfall_segment.instance_to(new_wf)
                collide()
                print("filled city :D")
                return task.done

        # check map boundaries
        x, y = round(_next_position.get_x()), round(_next_position.get_y())
        if x < -1 or x > MAP_SIZE - 2 or y < - 1 or y > MAP_SIZE - 2:
            print("collided with map boundary")
            collide()
            return task.done

        # check for collisions
        x, y = int(this_position.get_x()), int(this_position.get_y())
        aqueduct_segment_heights[y][x].append(this_position.get_z())
        x, y = int(_next_position.get_x()), int(_next_position.get_y())
        # other aqueducts
        for height in aqueduct_segment_heights[y][x]:
            if abs(height - _next_position.get_z()) < SEGMENT_HEIGHT*0.9: # give the player some leeway
                print("collided with aqueduct")
                collide()
                return task.done
            # columns
            if height > _next_position.get_z() and column_map[y][x]:
                print("collided with columns")
                collide()
                return task.done
        # terrain
        if _next_position.get_z() + 0.5 <= heightmap[(y + 1)//TILES_PER_BLOCK][(x + 1)//TILES_PER_BLOCK]:
            print("collided with map")
            collide()
            return task.done

        # follow with the camera
        LerpPosInterval(camera_pivot, BEAT_LENGTH, _next_position).start()
        camera_current_h = camera_pivot.get_h()
        camera_target_h = dir_h_dict[current_direction]
        if camera_current_h == 270 and camera_target_h == 0:
            LerpHprInterval(camera_pivot, BEAT_LENGTH / 2, (360, 0, 0), fluid=1).start()
        elif camera_current_h == 0 and camera_target_h == 270:
            LerpHprInterval(camera_pivot, BEAT_LENGTH / 2, (270, 0, 0), (360, 0, 0), fluid=1).start()
        elif camera_current_h == 360 and camera_target_h == 0:
            camera_pivot.set_h(0)
        else:
            LerpHprInterval(camera_pivot, BEAT_LENGTH/2, (dir_h_dict[current_direction], 0, 0), fluid=1).start()

        _action = None

        if is_down(speed_key):
            bl = BEAT_LENGTH / 4
        else:
            bl = BEAT_LENGTH
        task.delayTime = bl - (clock.getRealTime() % bl)
        return task.again


    def new_path(type):
        global _action
        global _next_position
        global current_direction
        _action = None
        if type == 'aq':
            waterfall.show()
            for _tank in filled_tanks:
                if _tank not in full_tanks:
                    tank = _tank
                    full_tanks.append(tank)
                    _next_position = base.render.get_relative_point(tank, (0, 2, 1.5))

                    current_direction = [k for k, v in dir_h_dict.items() if v == round(tank.get_h())][0]
                    break
            else:
                if len(lake_screws) == 0:
                    return end_game()
                random.shuffle(lake_screws)
                sc = lake_screws.pop()
                _next_position = Point3(
                    sc.get_x(),
                    sc.get_y() + 1,
                    sc.get_z()
                )
                current_direction = n
            #     if len(empty_tank_locs) == 0:
            #         # todo: spawn tank
            #         pass
            #     else:
            #         fill_tank(*empty_tank_locs.pop())
            #         tank = [t for t in filled_tanks if t not in full_tanks][0]
            # full_tanks.append(tank)
            camera_pivot.set_pos(_next_position)
            base.task_mgr.do_method_later(BEAT_LENGTH - (clock.getRealTime() % BEAT_LENGTH), build_segment, "build segment")
        elif type == 'r':
            base.task_mgr.do_method_later(BEAT_LENGTH - (clock.getRealTime() % BEAT_LENGTH), build_road, "build road")
        base.ignore('w')
        base.ignore('s')

    def collide():
        aqueduct.flatten_strong()
        waterfall.hide()

        # draw out a road that the player has to take back to the start
        new_path('aq')

        # todo: let user decide whether to do a road or aqueduct
        base.accept('w', new_path, ['aq'])
        # base.accept('s', new_path, ['r'])
        pass

    new_path('aq')


    road_segments = []

    def build_road(task):
        global _action
        global _next_position
        global current_direction
        this_position = _next_position
        this_h = dir_h_dict[current_direction]
        task.delayTime = BEAT_LENGTH/2
        is_down = base.mouseWatcherNode.is_button_down
        if _action == left_button or is_down(KeyboardButton.ascii_key(b'a')):
            # go left
            current_direction = (current_direction - 1) % 4
        elif _action == right_button or is_down(KeyboardButton.ascii_key(b'd')):
            # go right
            current_direction = (current_direction + 1) % 4
        else:
            # go straight
            pass
        new_seg = base.render.attach_new_node("segment " + str(len(segments)))
        segments.append(new_seg)
        x, y = round(this_position.get_x()), round(this_position.get_y())
        new_seg.set_pos(this_position)
        new_seg.set_z(heightmap[(y+1)//TILES_PER_BLOCK][(x+1)//TILES_PER_BLOCK])
        road_segment.instance_to(new_seg)
        road_segments.append(new_seg)

        # set up the next position
        x, y = round(_next_position.get_x()), round(_next_position.get_y())
        _next_position = this_position + dir_vec_dict[current_direction]

        # check map boundaries
        x, y = round(_next_position.get_x()), round(_next_position.get_y())
        if x < -1 or x > MAP_SIZE - 2 or y < -1 or y > MAP_SIZE - 2:
            collide()
            return task.done

        # todo: check collisions with buildings/columns/aqueduct

        # follow with the camera
        _next_position.set_z(new_seg.get_z())
        LerpPosInterval(camera_pivot, BEAT_LENGTH/2, _next_position).start()

        _action = None

        return task.again

    def end_game():
        def menu():
            import os
            os.system("python3 run_game.py")
            sys.exit()
            pass
        def retry():
            base.ignoreAll()
            base.render.node().removeAllChildren()
            base.render.node().clearState()
            end.hide()
            end_image.hide()
            main(base, SANDBOX, MAP_SIZE)
        screw_marker.hide()
        controls.hide()
        base.accept("m", menu)
        base.accept("r", retry)
        base.accept("x", sys.exit)
        tanks_filled = len(filled_tanks)
        total_tanks = len(empty_tank_locs)
        cities_satisfied = 3 - len(cities)

        end_message = "Your project is over.\nYou filled " + str(tanks_filled) + " out of " + str(total_tanks) + " tanks."
        if not SANDBOX:
            end_message += "\nYou supplied water to " + str(cities_satisfied) + " cities!"
        end_message += "\n\nYou can do better. Press r to try again.\nPress m to return to menu\nPress x to exit"

        end = OnscreenText(end_message, parent=base.aspect2d, pos=(0.5, 0), scale=0.1)
        end_image = OnscreenImage("bg.png", parent=base.render2d)


    def follow_cam(task):
        dlnp.set_pos(base.render, camera_pivot.get_pos(base.render) + LVector3(128, 128, 64))
        return task.cont

    base.task_mgr.add(follow_cam, "fc")

    base.render.set_shader_auto()
    base.render.setAntialias(AntialiasAttrib.MMultisample)
    base.shaderenable = 1
